@@ -49,7 +49,17 @@ typedef struct
 static area_id_t areas_[LNO_][ZNO_];
 static area_id_t *current_ = NULL;
 
-static GtkWidget *colorseldlg_ = NULL;                                                  
+static GtkWidget *colorseldlg_ = NULL;
+
+
+void
+mixgtk_colorsel_init (void)
+{
+  color_dialog_ = NULL;
+  colorseldlg_ = NULL;
+  current_ = NULL;
+}
+
 static void
 set_area_color_ (GtkWidget *area, const GdkColor *color)
 {
