@@ -26,11 +26,16 @@
 #define MIXGTK_CONFIG_H
 
 #include <mixlib/mix.h>
+#include <mixlib/mix_config.h>
 
 /* load configuration */
 extern gboolean
 mixgtk_config_load (void);
 
+/* get mix config */
+extern mix_config_t *
+mixgtk_config_get_mix_config (void);
+
 /* autosave state */
 extern gboolean
 mixgtk_config_is_autosave (void);
@@ -50,6 +55,13 @@ mixgtk_config_get (const gchar *key);
 extern void
 mixgtk_config_save (void);
 
+/* shared config params */
+extern gboolean
+mixgtk_config_is_split (void);
+
+extern void
+mixgtk_config_set_split (gboolean split);
+
 
 #endif /* MIXGTK_CONFIG_H */
 
