@@ -24,10 +24,13 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <mixlib/mix_vm_command.h>
 #include <mixlib/xmix_device.h>
 #include "mixgtk.h"
+#include "mixgtk_gen_handlers.h"
 #include "mixgtk_widgets.h"
 #include "mixgtk_config.h"
+#include "mixgtk_cmd_dispatcher.h"
 #include "mixgtk_device.h"
 
 #define BIN_DEV_COL_ 5
@@ -36,6 +39,17 @@ static const gchar *DEV_FORMAT_KEY_ = "Device.format";
 
 /* device container */
 static GtkNotebook *container_ = NULL;
+/* devdir dialog */
+static GtkWidget *devdir_dlg_ = NULL;
+static GtkEntry *devdir_entry_ = NULL;
+
+/** configuration stuff */
+#define LAST_BIN_DEV_   mix_dev_DISK_7
+static GtkWidget *devdlg_ = NULL;
+static GtkWidget *dtoggle_ = NULL;
+static GtkWidget *wtoggle_ = NULL;
+static GtkWidget *combo_ = NULL;
+
 static gint last_pos_ = 0;
 /* virtual machine */
 static mix_vm_t *vm_ = NULL;
@@ -283,7 +297,6 @@ static mix_device_t *
 mixgtk_device_new_ (mix_device_type_t type)
 {
   struct mixgtk_device_t *dev = NULL;
-  gchar *name = NULL;
   
   g_return_val_if_fail (type < mix_dev_INVALID, NULL);
 
@@ -297,10 +310,7 @@ mixgtk_device_new_ (mix_device_type_t type)
       ((struct mixgtk_bin_device_t *)dev)->last_insert = 0;
     }
 
-  name = g_strconcat (g_get_home_dir(), "/",
-		      MIXGTK_FILES_DIR, "/", DEF_NAMES_[type], NULL);
-  construct_device_with_name_ (&dev->device, type, name);
-  g_free (name);
+  construct_device_ (&dev->device, type);
 
   dev->device.vtable = &MIXGTK_VTABLE_;
 
@@ -309,20 +319,63 @@ mixgtk_device_new_ (mix_device_type_t type)
   return (mix_device_t *)dev;
 }
 
+static mix_device_t *
+mixgtk_device_copy_ (const struct mixgtk_device_t *from)
+{
+  struct mixgtk_device_t *result =
+    (struct mixgtk_device_t *)mixgtk_device_new_ (from->device.type);
+  g_return_val_if_fail (result != NULL, NULL);
+  if (MODES_[from->device.type] == mix_dev_CHAR)
+    {
+      gchar *text = gtk_editable_get_chars (GTK_EDITABLE (from->widget),
+					    0, -1);
+      gtk_text_insert (GTK_TEXT (result->widget), NULL, NULL, NULL, text, -1);
+      g_free (text);
+    }
+  else
+    {
+      static gchar *VALS[BIN_DEV_COL_] = {""};
+      struct mixgtk_bin_device_t *to = (struct mixgtk_bin_device_t *) result;
+      struct mixgtk_bin_device_t *fr = (struct mixgtk_bin_device_t *) from;
+      to->last_insert = fr->last_insert;
+      to->dec = fr->dec;
+      if (to->last_insert > 0)
+	{
+	  gint k, j, rows =  to->last_insert / BIN_DEV_COL_;
+	  GtkCList *tl = GTK_CLIST (result->widget);
+	  GtkCList *fl = GTK_CLIST (from->widget);
+	  for (k = 0; k < rows; ++k)
+	    {
+	      mix_word_t *words = g_new (mix_word_t, BIN_DEV_COL_);
+	      mix_word_t *oldwords =
+		(mix_word_t *) gtk_clist_get_row_data (fl, k);
+	      for (j = 0; j < BIN_DEV_COL_; ++j) words[j] = oldwords[j];
+	      gtk_clist_append (tl, VALS);
+	      gtk_clist_set_row_data_full (tl, k, words, g_free);
+	    }
+	  redraw_bin_device_ (to);
+	}
+    }
+  return (mix_device_t *)result;
+}
+
 /* init default devices */
 gboolean
 mixgtk_device_init (GtkNotebook *container, mix_vm_t *vm)
 {
-  static mix_device_type_t def_types[] = {
-    /* mix_dev_CONSOLE,*/ mix_dev_INVALID
-  };
-  
   gint k = 0;
   
   g_return_val_if_fail (container != NULL, FALSE);
   g_return_val_if_fail (vm != NULL, FALSE);
   container_ = container;
   vm_ = vm;
+  last_pos_ = 0;
+  devdlg_ = NULL;
+  devdir_dlg_ = NULL;
+  devdir_entry_ = NULL;
+  dtoggle_ = NULL;
+  wtoggle_ = NULL;
+  combo_ = NULL;
 
   /* remove dummy page from container */
   gtk_notebook_remove_page (container_, 0);
@@ -333,12 +386,16 @@ mixgtk_device_init (GtkNotebook *container, mix_vm_t *vm)
   if (mixgtk_config_get (DEV_FORMAT_KEY_))
     decs_ =  atoi (mixgtk_config_get (DEV_FORMAT_KEY_));
     
-  /* connect default devices */
-  while (def_types[k] != mix_dev_INVALID)
+  /* re-create existing devices */
+  for (k = 0; k < mix_dev_INVALID; ++k)
     {
-      struct mix_device_t *dev = mixgtk_device_new_ (def_types[k]);
-      if (dev != NULL)	mix_vm_connect_device (vm, dev);
-      ++k;
+      mix_device_t *dev = mix_vm_get_device (vm, k);
+      if (dev != NULL)
+	{
+	  mix_device_t *newdev =
+	    mixgtk_device_copy_ ((struct mixgtk_device_t *)dev);
+	  mix_vm_connect_device (vm, newdev);
+	}
     }
 
   /* set to first page */
@@ -367,13 +424,6 @@ mixgtk_device_set_format (mix_device_type_t dev, gboolean dec)
 }
 
 
-/** configuration stuff */
-#define LAST_BIN_DEV_   mix_dev_DISK_7
-static GtkWidget *devdlg_ = NULL;
-static GtkWidget *dtoggle_ = NULL;
-static GtkWidget *wtoggle_ = NULL;
-static GtkWidget *combo_ = NULL;
-
 static mix_device_type_t
 get_device_idx_ (void)
 {
@@ -446,6 +496,7 @@ on_devapp_button_clicked ()
   gchar value[20];
   for (k = 0; k <= LAST_BIN_DEV_; ++k)
     mixgtk_device_set_format (k, IS_DEC (new_decs_, k));
+  decs_ = new_decs_;
   snprintf (value, 20, "%d", decs_);
   mixgtk_config_update (DEV_FORMAT_KEY_, value);
 }
@@ -471,3 +522,48 @@ on_devset_button_clicked ()
     ON : OFF;
 }
 
+void
+on_devdir_activate ()
+{
+  static const gchar *DEVDIR_ENTRY_NAME = "devdir_entry";
+  if (devdir_dlg_ == NULL)
+    {
+      devdir_dlg_ = mixgtk_widget_factory_get_dialog (MIXGTK_DEVDIR_DIALOG);
+      g_assert (devdir_dlg_);
+      devdir_entry_ = GTK_ENTRY
+	(mixgtk_widget_factory_get_child_by_name (MIXGTK_DEVDIR_DIALOG,
+						  DEVDIR_ENTRY_NAME));
+      g_assert (devdir_entry_);
+    }
+  gtk_entry_set_text (devdir_entry_, mix_device_get_dir ());
+  gtk_widget_show (devdir_dlg_);
+}
+
+static void
+devdir_callback (const gchar *file)
+{
+  gtk_entry_set_text (devdir_entry_, file);
+}
+
+void
+on_devdir_browse_clicked ()
+{
+  mixgtk_get_file (devdir_callback, "Devices dir", mix_device_get_dir ());
+}
+
+void
+on_devdir_cancel_clicked ()
+{
+  gtk_widget_hide (devdir_dlg_);
+}
+
+void
+on_devdir_ok_clicked ()
+{
+  const gchar *dirname = gtk_entry_get_text (devdir_entry_);
+  gchar *cmd = g_strconcat (mix_vm_command_to_string (MIX_CMD_DEVDIR),
+			    " ", dirname, NULL);
+  gtk_widget_hide (devdir_dlg_);
+  mixgtk_cmd_dispatcher_dispatch (cmd);
+  g_free (cmd);
+}
