@@ -42,7 +42,6 @@ on_file_edit_activate (GtkWidget *w, gpointer data);
 extern void
 on_file_compile_activate (GtkWidget *w, gpointer data);
 
-
 extern void
 on_clear_breakpoints_activate (GtkWidget *w, gpointer data);
 
@@ -52,6 +51,11 @@ on_save_on_exit_toggle (GtkWidget *w, gpointer data);
 extern void
 on_save_activate (GtkWidget *w, gpointer data);
 
+typedef void (*file_callback_t)(const gchar *file);
+
+extern void
+mixgtk_get_file (file_callback_t callback, const gchar *title,
+		 const gchar *pattern);
 
 
 #endif /* MIXGTK_GEN_HANDLERS_H */
