@@ -32,7 +32,7 @@
 
 #define MIXAL_TAB_POS_ 2
 
-static mix_vm_t *vm_;
+static mix_vm_t *vm_ = NULL;
 static GtkCList *clist_;
 static gulong lineno_;
 static GtkStatusbar *status_;
@@ -128,19 +128,23 @@ init_color_ (GdkColor *c, const gchar *name)
 
 /* initialise the mixal widgets */
 gboolean
-mixgtk_mixal_init (mix_vm_t *vm)
+mixgtk_mixal_init (mix_vm_t *vm, mixgtk_dialog_id_t top)
 {
+  static gboolean restart = FALSE;
   int i,j;
   
   g_return_val_if_fail (vm != NULL, FALSE);
   vm_ = vm;
-  clist_ = GTK_CLIST (mixgtk_widget_factory_get (MIXGTK_WIDGET_MIXAL));
+  clist_ = GTK_CLIST (mixgtk_widget_factory_get (top, MIXGTK_WIDGET_MIXAL));
   g_return_val_if_fail (clist_ != NULL, FALSE);
   
-  status_ = GTK_STATUSBAR (mixgtk_widget_factory_get (MIXGTK_WIDGET_STATUSBAR));
+  status_ = GTK_STATUSBAR
+    (mixgtk_widget_factory_get (MIXGTK_MAIN, MIXGTK_WIDGET_STATUSBAR));
   g_return_val_if_fail (status_ != NULL, FALSE);
   status_context_ = gtk_statusbar_get_context_id (status_, "MIXAL status");
   
+  symbols_dlg_ = symbols_clist_ = NULL;
+
   /* allocate colors */
   colormap_ = gtk_widget_get_colormap (GTK_WIDGET (clist_));
   for (i = 0; i < 3; ++i)
@@ -159,6 +163,10 @@ mixgtk_mixal_init (mix_vm_t *vm)
 				  FALSE);
 	  }
       }
+
+  if (restart) mixgtk_mixal_load_file ();
+  else restart = TRUE;
+  
   return TRUE;
 }
 
@@ -390,12 +398,15 @@ on_mixal_motion_notify_event (GtkWidget *list, GdkEventMotion *event,
   if (gtk_clist_get_selection_info (clist_, event->x, event->y, &row, &col)
       && row != last_row && tips_text_)
     {
+      const gchar *msg = NULL;
       last_row = row;
       if (last_message)
 	gtk_statusbar_remove (status_, status_context_, last_message);
-      last_message = gtk_statusbar_push
-	(status_, status_context_,
-	 (const gchar *)g_ptr_array_index (tips_text_, row));
+      msg = (const gchar *)g_ptr_array_index (tips_text_, row);
+      if (msg)
+	last_message = gtk_statusbar_push (status_, status_context_, msg);
+      else
+	last_message = 0;
      }
   return FALSE;
 }
@@ -420,3 +431,9 @@ on_notebook_switch_page (GtkNotebook *notebook)
   if (p != MIXAL_TAB_POS_) gtk_statusbar_pop (status_, status_context_);
   if (p == MIXAL_TAB_POS_) mixgtk_mixal_update ();
 }
+
+void
+on_mixal_leave_notify_event ()
+{
+  gtk_statusbar_pop (status_, status_context_);
+}
