@@ -27,10 +27,11 @@
 
 #include <mixlib/mix_vm.h>
 #include "mixgtk.h"
+#include "mixgtk_widgets.h"
 
 /* initialise the mixvm widgets */
 extern gboolean
-mixgtk_mixvm_init (mix_vm_t *vm);
+mixgtk_mixvm_init (mix_vm_t *vm, mixgtk_dialog_id_t top);
 
 /* update register widgets */
 extern void
