@@ -65,7 +65,14 @@ vm_reset_reload_ (mix_vm_t *vm, gboolean is_reload)
 	mix_device_delete (vm->devices[k]);
 	vm->devices[k] = NULL;
       }
-  if (!is_reload) bp_clear_all_ (vm);
+  if (!is_reload)
+    mix_vm_clear_all_breakpoints (vm);
+
+  if (vm->address_list)
+    {
+      g_slist_free (vm->address_list);
+      vm->address_list = NULL;
+    }
 }
 
 
@@ -80,6 +87,8 @@ mix_vm_new (void)
   vm->address_table = NULL;
   vm->symbol_table = NULL;
   vm->src_file = NULL;
+  vm->pred_list = mix_predicate_list_new (vm);
+  vm->address_list = NULL;
   
   for (i = 0; i < BD_NO_; ++i)
     vm->devices[i] = NULL;
@@ -104,6 +113,8 @@ mix_vm_delete (mix_vm_t * vm)
   if (vm->address_table != NULL) g_tree_destroy (vm->address_table);
   if (vm->symbol_table != NULL) mix_symbol_table_delete (vm->symbol_table);
   if (vm->src_file != NULL) mix_src_file_delete (vm->src_file);
+  if (vm->pred_list != NULL) mix_predicate_list_delete (vm->pred_list);
+  if (vm->address_list != NULL) g_slist_free (vm->address_list);
   for (i = 0; i < BD_NO_; ++i)
     mix_device_delete (vm->devices[i]);
   mix_vm_clock_delete (vm->clock);
@@ -416,25 +427,6 @@ mix_vm_get_lineno_address (const mix_vm_t *vm, guint lineno)
 }
 
 
-/* Reposition program counter and reset state so that a loaded
-   program can be restarted.
-*/
-void
-mix_vm_reset_program (mix_vm_t *vm)
-{
-  guint k;
-  g_return_if_fail (vm != NULL);
-  reset_loc_ (vm);
-  halt_ (vm, FALSE);
-  set_rA_ (vm, MIX_WORD_ZERO);
-  set_rX_ (vm, MIX_WORD_ZERO);
-  set_rJ_ (vm, MIX_WORD_ZERO);
-  for ( k = 1; k < IREG_NO_+1; ++k )
-    set_rI_ (vm, k, MIX_WORD_ZERO);
-  set_over_ (vm, FALSE);
-  set_cmp_ (vm, mix_EQ);
-}
-
 /* continue execution of instructions in memory */
 int
 mix_vm_run (mix_vm_t *vm)
@@ -445,12 +437,17 @@ mix_vm_run (mix_vm_t *vm)
   while ( !is_halted_ (vm) )
     {
       mix_word_to_ins_uncheck (get_cell_ (vm, get_loc_ (vm)), ins);
+      vm->address_list =
+	g_slist_prepend (vm->address_list,
+			 GINT_TO_POINTER ((gint)get_loc_ (vm)));
       if ( !(*ins_handlers_[ins.opcode]) (vm,&ins) ) 
 	return MIX_VM_ERROR;
       else
 	update_time_ (vm, &ins);
       if (bp_is_set_ (vm, get_loc_ (vm)))
 	return MIX_VM_BREAK;
+      if (mix_predicate_list_eval (get_pred_list_ (vm)))
+	return MIX_VM_COND_BREAK;
       if (get_loc_ (vm) >= MIX_VM_CELL_NO) halt_ (vm, TRUE);
     }
   return MIX_VM_HALT;
@@ -464,13 +461,19 @@ mix_vm_exec_next (mix_vm_t *vm)
   g_return_val_if_fail (vm != NULL, MIX_VM_ERROR);
   if (get_loc_ (vm) >= MIX_VM_CELL_NO) halt_ (vm, TRUE);
   if (is_halted_ (vm)) return MIX_VM_HALT;
+  vm->address_list =
+    g_slist_prepend (vm->address_list,
+		     GINT_TO_POINTER ((gint)get_loc_ (vm)));
   mix_word_to_ins_uncheck (get_cell_ (vm, get_loc_ (vm)), ins);
   if (!(*ins_handlers_[ins.opcode]) (vm, &ins))
     return MIX_VM_ERROR;
   else
     update_time_ (vm, &ins);
   if (is_halted_ (vm)) return MIX_VM_HALT;
-  return bp_is_set_ (vm, get_loc_ (vm)) ? MIX_VM_BREAK : MIX_VM_OK;
+  if (bp_is_set_ (vm, get_loc_ (vm))) return MIX_VM_BREAK;
+  if (mix_predicate_list_eval (get_pred_list_ (vm)))
+    return MIX_VM_COND_BREAK;
+  return MIX_VM_OK;
 }
 
 /* Breakpoints */
@@ -590,6 +593,35 @@ mix_vm_clear_all_breakpoints (mix_vm_t *vm)
 {
   g_return_if_fail (vm != NULL);
   bp_clear_all_ (vm);
+  mix_predicate_list_clear (get_pred_list_ (vm));
+}
+
+
+gboolean
+mix_vm_set_conditional_breakpoint (mix_vm_t *vm, mix_predicate_t *pred)
+{
+  g_return_val_if_fail (vm != NULL, FALSE);
+  g_return_val_if_fail (pred != NULL, FALSE);
+  mix_predicate_list_add (get_pred_list_ (vm), pred);
+  return TRUE;
+}
+
+gboolean
+mix_vm_clear_conditional_breakpoint (mix_vm_t *vm, mix_predicate_t *pred)
+{
+  g_return_val_if_fail (vm != NULL, FALSE);
+  g_return_val_if_fail (pred != NULL, FALSE);
+  return mix_predicate_list_remove (get_pred_list_ (vm), pred);
+}
+
+const gchar *
+mix_vm_get_last_breakpoint_message (const mix_vm_t *vm)
+{
+  const mix_predicate_t *last = NULL;
+  g_return_val_if_fail (vm != NULL, NULL);
+  if ((last = mix_predicate_list_last_true_eval (get_pred_list_ (vm))) != NULL)
+    return mix_predicate_get_message (last);
+  return NULL;
 }
 
 /* Get the vm uptime, defined as the time spent executing instructions */
@@ -600,3 +632,10 @@ mix_vm_get_uptime (const mix_vm_t *vm)
   return mix_vm_clock_get_time (get_clock_ (vm));
 }
 
+/* Get the list of addresses for executed instructions */
+const GSList *
+mix_vm_get_backtrace (const mix_vm_t *vm)
+{
+  g_return_val_if_fail (vm != NULL, NULL);
+  return get_address_list_ (vm);
+}
