@@ -151,17 +151,12 @@ mix_vm_get_address_lineno (const mix_vm_t *vm, mix_address_t addr);
 extern mix_address_t
 mix_vm_get_lineno_address (const mix_vm_t *vm, guint lineno);
 
-/* Reposition program counter and reset state so that a loaded
-   program can be restarted.
-*/
-extern void
-mix_vm_reset_program (mix_vm_t *vm);
-
 /* continue execution of instructions in memory */
 /* Possible outcomes */
 enum {
   MIX_VM_ERROR,		/* error executing instructions */
   MIX_VM_BREAK,		/* breakpoint found */
+  MIX_VM_COND_BREAK,    /* conditional breakpoint found */
   MIX_VM_HALT,		/* end of execution */
   MIX_VM_OK		/* successful instruction execution */
 };
@@ -205,10 +200,24 @@ mix_vm_has_breakpoint_at_address (const mix_vm_t *vm, guint address);
 extern void
 mix_vm_clear_all_breakpoints (mix_vm_t *vm);
 
+#include "mix_predicate.h"
+extern gboolean
+mix_vm_set_conditional_breakpoint (mix_vm_t *vm, mix_predicate_t *pred);
+
+extern gboolean
+mix_vm_clear_conditional_breakpoint (mix_vm_t *vm, mix_predicate_t *pred);
+
+extern const gchar *
+mix_vm_get_last_breakpoint_message (const mix_vm_t *vm);
+
 /* Get the vm uptime, defined as the time spent executing instructions */
 extern mix_time_t
 mix_vm_get_uptime (const mix_vm_t *vm);
 
+/* Get the list of addresses for executed instructions */
+extern const GSList *
+mix_vm_get_backtrace (const mix_vm_t *vm);
+
 
 #endif /* MIX_VM_H */
 
