@@ -28,6 +28,9 @@
 #include "mix_file.h"
 #include "mix_device.h"
 
+/* device file directory */
+extern gchar *DEV_DIR_;
+
 /* table of overridable device operations */
 typedef gboolean (*mix_dev_write_func_t) (mix_device_t *, const mix_word_t *);
 typedef gboolean (*mix_dev_read_func_t) (mix_device_t *, mix_word_t *);
