@@ -29,6 +29,7 @@
 #include "mix_symbol_table.h"
 #include "mix_device.h"
 #include "mix_src_file.h"
+#include "mix_predicate_list.h"
 #include "mix_vm.h"
 
 /* The mix_vm_t type */
@@ -50,6 +51,8 @@ struct mix_vm_t
   mix_symbol_table_t *symbol_table;
   mix_src_file_t *src_file;	/* source of last loaded code file */
   mix_device_factory_t factory; /* the factory for new devices */
+  mix_predicate_list_t *pred_list; /* predicates for conditional bps */
+  GSList *address_list;		/* list of executed addresses */
 };
 
 /* Macros for accessing/modifying the above structure.
@@ -72,6 +75,8 @@ enum { A_ = 0, X_, J_, I1_, I2_, I3_, I4_, I5_, I6_ };
 #define get_over_(vm) (vm->overflow)
 #define get_loc_(vm)  (vm->loc_count)
 #define get_clock_(vm) (vm->clock)
+#define get_pred_list_(vm) (vm->pred_list)
+#define get_address_list_(vm) (vm->address_list)
 
 #define set_reg_(vm,r,x)			\
 do {						\
