@@ -37,7 +37,7 @@
 extern void
 mix_vmloop (const gchar *code_file, gboolean use_emacs);
 
-static void
+extern void
 mix_vmrun (const gchar *code_file, gboolean dump);
 
 enum {
@@ -137,24 +137,5 @@ main (int argc, char **argv)
 }
 
 
-static void
-mix_vmrun (const gchar *code_file, gboolean dump) 
-{
-  mix_vm_t *vm = mix_vm_new ();
-  if (!mix_vm_load_file (vm, code_file)) {
-    fprintf (stderr, _("Error loading %s file\n"), code_file);
-    return;
-  }
-  mix_vm_run (vm);
-  printf (_("** Execution time: %ld\n"), mix_vm_get_uptime (vm));
-  if (dump) {
-    mix_dump_context_t *dc = mix_dump_context_new (MIX_DUMP_DEF_CHANNEL,
-						   0, 0,
-						   MIX_DUMP_ALL_NOMEM);
-    mix_vm_dump (vm, dc);
-    mix_dump_context_delete (dc);
-  }
-  mix_vm_delete (vm);
-}
 
   
