@@ -30,7 +30,10 @@
 #ifdef HAVE_LIBREADLINE
 #  include <readline/readline.h>
 #  include <readline/history.h>
-#else
+#  ifndef HAVE_RL_COMPLETION_MATCHES /* old versions of rl don't use rl_ */
+#    define rl_completion_matches completion_matches
+#  endif
+#else  /* ! HAVE_LIBREADLINE */
   typedef int Function ();
 #endif /* HAVE_LIBREADLINE */
 
@@ -76,7 +79,7 @@ static const char *mix_commands_[ALL_COMMANDS_NO_] = {NULL};
 #ifdef HAVE_LIBREADLINE
 /* readline functions */
 static char *
-mixvm_cmd_generator_ (char *text, int state);
+mixvm_cmd_generator_ (const char *text, int state);
 
 
 /* Attempt to complete on the contents of TEXT.  START and END bound the
@@ -95,7 +98,7 @@ mixvm_cmd_completion_ (char *text, int start, int end)
      to complete.  Otherwise it is the name of a file in the current
      directory. */
   if (start == 0)
-    matches = completion_matches (text, mixvm_cmd_generator_);
+    matches = rl_completion_matches (text, mixvm_cmd_generator_);
   
   return (matches);
 }
@@ -104,7 +107,7 @@ mixvm_cmd_completion_ (char *text, int start, int end)
    to start from scratch; without any state (i.e. STATE == 0), then we
    start at the top of the list. */
 static char *
-mixvm_cmd_generator_ (char *text, int state)
+mixvm_cmd_generator_ (const char *text, int state)
 {
   static int list_index, len;
   const char *name;
@@ -173,33 +176,23 @@ cmd_help_ (char *arg)
   int i;
   int printed = 0;
      
-  for (i = 0; commands[i].name; i++)
+  for (i = 1; commands[i].name; i++)
     {
-      if (!*arg || (strcmp (arg, commands[i].name) == 0))
+      if (!arg || !*arg)
 	{
-	  printf (_("%s\t\t%s. Usage: %s\n"), commands[i].name, 
+	  printf (_("%s\t\t%s.\n"), commands[i].name, _(commands[i].doc));
+	  printed++;
+	}
+      else if (strcmp (arg, commands[i].name) == 0)
+	{
+	  printf (_("%s\t\t%s.\nUsage:\t\t%s\n"), commands[i].name, 
 		  _(commands[i].doc), commands[i].usage);
 	  printed++;
 	}
     }
-
-  if (printed > 1) printf ("\n");
-
-  for (i = LOCAL_COMANDS_NO_ + 1 /* skip help cmd */; mix_commands_[i]; i++)
-    {
-      if (!*arg || (strcmp (arg, mix_commands_[i]) == 0))
-	{
-	  printf (_("%s\t\t%s. Usage: %s\n"), mix_commands_[i], 
-		  mix_vm_command_help (i - LOCAL_COMANDS_NO_),
-		  mix_vm_command_usage (i - LOCAL_COMANDS_NO_));
-	  printed++;
-	}
-    }
-  
-  if (!printed) printf ("Command \'%s\' not found\n", arg);
-  
+  if (!printed ||!arg || !*arg)
+    mix_vm_cmd_dispatcher_dispatch (dis_, MIX_CMD_HELP, arg);
   return TRUE;
-  
 }
 
 static int
@@ -246,7 +239,7 @@ init_dis_ (mix_vm_cmd_dispatcher_t *dis)
 }
 
 void
-mixvm_cmd_init (char *arg, gboolean use_emacs)
+mixvm_cmd_init (mix_config_t *config, char *arg, gboolean use_emacs)
 {
   int k;
   /* get local command names */
@@ -263,7 +256,7 @@ mixvm_cmd_init (char *arg, gboolean use_emacs)
 #endif /* HAVE_LIBREADLINE */
 
   /* initialise the dispatcher */
-  dis_ = mix_vm_cmd_dispatcher_new (stdout, stderr);
+  dis_ = mix_vm_cmd_dispatcher_new_with_config (stdout, stderr, config);
 
   if ( dis_ == NULL)
     g_error (_("Failed initialisation (no memory resources)"));
